import traceback
import queue
import time
import atexit
import threading

class HandleOutput:

    def __init__(self, output_config, batch_config, file):
        self.terminal_output = output_config["terminal"]
        self.file_output = output_config["file"]
        self.batch_config_toggle = batch_config["batch_logging"]
        self.log_buffer_max_size = batch_config["batch_size"]
        self.flush_interval = batch_config["flush_interval"]

        self.file_path = file

        self._exit_flag = False

        self.log_buffer = []
        self.last_flush_time = time.time()

        self.log_queue = queue.Queue()

        self.bg_task = threading.Thread(target=self._process_logs, daemon=True)
        self.bg_task.start()

        

        atexit.register(self._flush_on_exit)

    def _flush_on_exit(self):
        self.flush(override=True)
        self._exit_flag = True
        self.log_queue.put(None)

        self.bg_task.join()

    def _process_logs(self):
        
        while not self._exit_flag:
            message = self.log_queue.get(timeout=3)
            if message is None:
                break
            self.handle_log_output(message)


    def handle_log_output(self, message):

        if self.batch_config_toggle:

            if self.check_batching_condition():

                self.flush()

            else:

                    self.log_buffer.append(message)
        else:

            if self.terminal_output:

                print(message, flush=True)

            if self.file_output:

                    with open(self.file_path, 'a', encoding='utf-8') as log_file:
                        log_file.write(f"{message}\n")

    def flush(self, override= False):

        if self.log_buffer or override:

            with open(self.file_path, 'a', encoding='utf-8') as log_file:
                log_file.write('\n'.join(self.log_buffer) + '\n')

            self.log_buffer.clear()
            self.last_flush_time = time.time()
            self.log_internal_message(f"Flushed {self.log_buffer_max_size} logs")

    def check_batching_condition(self):

        if len(self.log_buffer) >= self.log_buffer_max_size \
            or time.time() - self.last_flush_time >= self.flush_interval:
            return True
        return False

    @staticmethod
    def log_internal_message(message, exception_traceback=None):

        print(f"\033[1;37m[INTERNAL] {message}")

        if exception_traceback:

            print(
                f"{traceback.format_exception(None, exception_traceback, exception_traceback.__traceback__)}")
